@@ -8,6 +8,7 @@ import pickle
 import numpy as numpy
 import os
 
+
 class CustomDataset(Dataset):
     def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None):
         self.root_dir = root_dir
@@ -42,6 +43,7 @@ class CustomDataset(Dataset):
         direction = np.array(direction).reshape(3,3)
         inverse = np.where(direction == -1)[0]
         permute = np.where(direction != 0)[1]
+        img = img.transpose(permute)
         if 0 in inverse:
             img = img[::-1, :, :]
 
@@ -52,21 +54,6 @@ class CustomDataset(Dataset):
             img = img[:, :, ::-1]
 
         return img.transpose(permute)
-
-    def get_maximum_patch_size(self):
-
-        result = torch.tensor((0,0,0))
-        for idx in range(self.__len__()):
-            if self.mode == 'train':
-                x,  _, _ =  self.__getitem__(idx)
-            
-            if self.mode == 'test':
-                x,  _ =  self.__getitem__(idx)
-
-            shape = torch.tensor((x.shape[-3], x.shape[-2], x.shape[-1]))
-            result = torch.max(torch.vstack((result, shape)), dim = 0)[0]
-
-        return result
             
     def __getitem__(self, idx):
         info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
@@ -135,6 +122,181 @@ class CustomDataset(Dataset):
             
 
 
+class CustomDatasetV2(CustomDataset):
+    def __init__(self, root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None,
+                 patch_size = (128, 128, 128), transform = None, batch_size = 2):
+        
+        self.patch_size = np.array(patch_size)
+        self.transform = transform
+        self.batch_size = batch_size
+        super().__init__(root_dir, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None)
+    
+    def __len__(self):
+        return super.__len__()
+    
+    def set_direction(self, img, direction):
+        return super().set_direction(img, direction)
+    
+    def get_oversample_patch(self, images, seg):
+        if not isinstance(images, np.ndarray):
+            images = images.numpy()
+
+        if not isinstance(seg, np.ndarray):
+            seg = seg.numpy()
+
+        non_zero_coords = np.array(np.where(seg != 0))
+        shape_to_index = np.array(images[0].shape) - 1 
+        
+        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
+
+        oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
+        oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
+        oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
+
+        oversample_patch_images = images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        oversample_patch_seg = seg[oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        
+        return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
+    
+    def get_normal_patch(self, images, seg):
+        if not isinstance(images, np.ndarray):
+            images = images.numpy()
+
+        if not isinstance(seg, np.ndarray):
+            seg = seg.numpy()
+
+        shape_to_index = np.array(images[0].shape) - 1 
+        normal_center = np.random.randint(low = self.patch_size/2 - 1, high = shape_to_index - (self.patch_size/2) + 1 )
+        normal_patch_upper_bound = normal_center + (self.patch_size/2) + 1
+        normal_patch_lower_bound = normal_center - (self.patch_size/2 - 1)
+        normal_patch_bound = np.concatenate((normal_patch_lower_bound.reshape(-1,1),normal_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
+        normal_patch_images = self.images[:, normal_patch_bound[0][0] : normal_patch_bound[0][1],
+                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
+                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
+        normal_patch_seg = self.seg[normal_patch_bound[0][0] : normal_patch_bound[0][1],
+                                          normal_patch_bound[1][0] : normal_patch_bound[1][1],
+                                          normal_patch_bound[2][0] : normal_patch_bound[2][1]]
+        
+        return np.expand_dims(normal_patch_images , axis = 0), np.expand_dims(normal_patch_seg , axis = 0)
+
+    """
+    def get_oversample_patch(self, images, seg):
+        non_zero_coords = torch.where(seg != 0)
+        shape_to_index = torch.tensor(images[0].shape) - 1
+        oversample_max_index = torch.max(torch.vstack(non_zero_coords), axis = 1)[0][:-3] #class가 있는 부분의 최대 index
+        oversample_min_index = torch.min(torch.vstack(non_zero_coords), axis = 1)[0][:-3] #class가 있는 부분의 최소 index
+        self.front_patch_size = self.patch_size//2
+        self.back_patch_size = shape_to_index - (self.patch_size//2 + self.patch_size%2)
+
+        oversample_center_upper_bound = torch.min((oversample_max_index, self.back_patch_size))
+    
+        oversample_center_x = torch.randint(low = oversample_min_index[-3], high = oversample_max_index[-3]) 
+        oversample_center_y = torch.randint(low = oversample_min_index[-2], high = oversample_max_index[-2])
+        oversample_center_z = torch.randint(low = oversample_min_index[-1], high = oversample_max_index[-1])
+
+        oversample_high_x = self.patch_size 
+        
+        oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+        oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
+
+        oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
+        oversample_patch_lower_bound = oversample_center - (self.patch_size/2 - 1)
+        oversample_patch_bound = np.concatenate((oversample_patch_lower_bound.reshape(-1,1),oversample_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
+
+        oversample_patch_images = images[:, oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        oversample_patch_seg = seg[oversample_patch_bound[0][0] : oversample_patch_bound[0][1],
+                                          oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
+                                          oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
+        
+        return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
+    """
+    
+    def __getitem__(self, idx):
+        info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
+        info_data = pickle.load(info_file)
+        info_file.close()
+        data = np.load(os.path.join(self.root_dir, self.image_name[idx]))
+        
+
+        x = data['x']
+        y = data['y']
+        if len(x.shape) == 3 and self.dim == '3d':
+            #modality가 1개인 경우
+            x = np.expand_dims(x, axis = 0)
+        elif len(x.shape) == 2 and self.dim == '2d':
+            x = np.expand_dims(x, axis = 0)
+
+        if self.correct_direction:
+            for i in range(len(x)):
+                x[i] = self.set_direction(x[i], info_data['direction'])
+            
+            if self.mode == 'train' :
+                y = self.set_direction(y, info_data['direction'])
+
+        if self.normalizer:
+            x = self.normalizer(x)
+        
+        
+        if self.resample != None:
+            if self.mode == 'train' :
+                y = torch.tensor(y.copy())
+                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
+                x = result[:-1]
+                y = result[-1].squeeze(0)
+            
+            else:
+                x = self.resample(x, info_data)
+        
+
+        if self.batch_size == 1 :
+            result_images, result_seg = self.get_normal_patch(x, y)
+
+
+        for i_b in range(self.batch_size//2):
+            oversample_images, oversample_seg = self.get_oversample_patch(x, y)
+            normal_images, normal_seg = self.get_normal_patch(x, y)
+
+            if i_b == 0:
+                result_images = np.concatenate((normal_images, oversample_images), axis = 0) #bs, n, x, y, z
+                result_seg = np.concatenate((normal_seg, oversample_seg), axis = 0) #bs, x, y, z
+            
+            else:
+                result_images = np.concatenate((result_images, oversample_images), axis = 0)
+                result_seg = np.concatenate((result_seg, oversample_seg), axis = 0)
+                result_images = np.concatenate((result_images, normal_images), axis = 0)
+                result_seg = np.concatenate((result_seg, normal_seg), axis = 0)
+
+
+        if isinstance(result_images, np.ndarray):
+            result_images = torch.tensor(result_images)
+        
+        if isinstance(result_seg, np.ndarray):
+            result_seg = torch.tensor(result_seg)
+        
+        if result_images.device.index != self.device:
+            result_images = result_images.to(self.device)
+
+        if result_seg.device.index != self.device:
+            result_seg = result_seg.to(self.device)
+
+        if self.transform:
+            result_images, result_seg = self.transform(result_images, result_seg, self.info)
+        
+
+        return result_images, result_seg.long()
+
+    
+
+
 
 class DataLoader3D(object):
 
