@@ -164,7 +164,7 @@ class Interactive_Trainer(Trainer):
                 images, seg = self.tr_dl.generate_train_batch()
                 bs, _, nx, ny, nz = images.shape #bs, n_modalities, 
                 #First
-                positive_points, negative_points = point_sampler(seg)
+                positive_points, negative_points = point_sampler(seg, first = True)
                 output = self.model(images, positive_points, negative_points)
                 self.optimizer.zero_grad()
                 loss = self.calculate_loss(output, seg, writer, e)
@@ -185,7 +185,7 @@ class Interactive_Trainer(Trainer):
                     prev_seg = output.detach()
                     print('prev_seg', time.time()- prev_seg_time)
                     ps_time = time.time()
-                    positive_points, negative_points = point_sampler(seg, prev_seg, positive_points, negative_points)
+                    point_sampler(seg, prev_seg, positive_points, negative_points)
                     out_time = time.time()
                     print('ps time', time.time()- ps_time)
                     output = self.model(images, positive_points, negative_points, prev_seg)
